@@ -96,6 +96,10 @@ void osl_relation_list_pprint_elts(FILE *, osl_relation_list_p, osl_names_p);
 void osl_relation_list_pprint(FILE *, osl_relation_list_p, osl_names_p);
 void osl_relation_list_print(FILE *, osl_relation_list_p);
 
+// SCoPLib Compatibility
+void osl_relation_list_pprint_access_array_scoplib(FILE *, osl_relation_list_p,
+                                                    osl_names_p, int);
+
 
 /*****************************************************************************
  *                               Reading function                            *
