@@ -67,6 +67,7 @@
 
 #include <osl/macros.h>
 #include <osl/util.h>
+#include <osl/extensions/arrays.h>
 #include <osl/extensions/textual.h>
 #include <osl/strings.h>
 #include <osl/relation.h>
@@ -217,8 +218,11 @@ osl_names_p osl_scop_names(osl_scop_p scop) {
  */
 void osl_scop_print(FILE * file, osl_scop_p scop) {
   int parameters_backedup = 0;
+  int arrays_backedup = 0;
   osl_strings_p parameters_backup = NULL;
+  osl_strings_p arrays_backup = NULL;
   osl_names_p names;
+  osl_arrays_p arrays;
 
   if (scop == NULL) {
     fprintf(file, "# NULL scop\n");
@@ -230,21 +234,28 @@ void osl_scop_print(FILE * file, osl_scop_p scop) {
   }
 
   if (osl_scop_integrity_check(scop) == 0)
-    OSL_warning("OpenScop integrity check failed. "
-                     "Something may go wrong.");
+    OSL_warning("OpenScop integrity check failed. Something may go wrong.");
   
   // Generate the names for the various dimensions.
   names = osl_scop_names(scop);
 
   while (scop != NULL) {
-    // If possible, replace parameter names with scop iterator names.
+    // If possible, replace parameter names with scop parameter names.
     if (osl_generic_has_URI(scop->parameters, OSL_URI_STRINGS)) {
       parameters_backedup = 1;
       parameters_backup = names->parameters;
       names->parameters = scop->parameters->data;
     }
-    
-    fprintf(file, "\n"OSL_TAG_START_SCOP"\n\n");
+
+    // If possible, replace array names with arrays extension names.
+    arrays = osl_generic_lookup(scop->extension, OSL_URI_ARRAYS);
+    if (arrays != NULL) {
+      arrays_backedup = 1;
+      arrays_backup = names->arrays;
+      names->arrays = osl_arrays_to_strings(arrays);
+    }
+
+    fprintf(file, "\n<"OSL_URI_SCOP">\n\n");
     fprintf(file, "# =============================================== "
                   "Global\n");
     fprintf(file, "# Language\n");
@@ -269,7 +280,7 @@ void osl_scop_print(FILE * file, osl_scop_p scop) {
                     "Extensions\n");
       osl_generic_print(file, scop->extension);
     }
-    fprintf(file, "\n"OSL_TAG_END_SCOP"\n\n");
+    fprintf(file, "\n</"OSL_URI_SCOP">\n\n");
     
     // If necessary, switch back parameter names.
     if (parameters_backedup) {
@@ -277,6 +288,113 @@ void osl_scop_print(FILE * file, osl_scop_p scop) {
       names->parameters = parameters_backup;
     }
 
+    // If necessary, switch back array names.
+    if (arrays_backedup) {
+      arrays_backedup = 0;
+      osl_strings_free(names->arrays);
+      names->arrays = arrays_backup;
+    }
+
+    scop = scop->next;
+  }
+
+  osl_names_free(names);
+}
+
+
+/**
+ * osl_scop_print_scoplib function:
+ * this function prints the content of an osl_scop_t structure (*scop)
+ * into a file (file, possibly stdout) in the ScopLib textual format.
+ * \param file The file where the information has to be printed.
+ * \param scop The scop structure whose information has to be printed.
+ */
+void osl_scop_print_scoplib(FILE * file, osl_scop_p scop) {
+  int parameters_backedup = 0;
+  int arrays_backedup = 0;
+  osl_strings_p parameters_backup = NULL;
+  osl_strings_p arrays_backup = NULL;
+  osl_names_p names;
+  osl_arrays_p arrays;
+
+  if (scop == NULL) {
+    fprintf(file, "# NULL scop\n");
+    return;
+  }
+  else {
+    fprintf(file, "# [File generated by the OpenScop Library %s]\n"
+                  "# [SCoPLib format]\n",
+                  OSL_RELEASE);
+  }
+
+  if (osl_scop_check_compatible_scoplib(scop) == 0) {
+    OSL_error("SCoP integrity check failed. Something may go wrong.");
+    exit(1);
+  }
+  
+  // Generate the names for the various dimensions.
+  names = osl_scop_names(scop);
+
+  while (scop != NULL) {
+    // If possible, replace parameter names with scop parameter names.
+    if (osl_generic_has_URI(scop->parameters, OSL_URI_STRINGS)) {
+      parameters_backedup = 1;
+      parameters_backup = names->parameters;
+      names->parameters = scop->parameters->data;
+    }
+
+    // If possible, replace array names with arrays extension names.
+    arrays = osl_generic_lookup(scop->extension, OSL_URI_ARRAYS);
+    if (arrays != NULL) {
+      arrays_backedup = 1;
+      arrays_backup = names->arrays;
+      names->arrays = osl_arrays_to_strings(arrays);
+    }
+
+    fprintf(file, "\nSCoP\n\n");
+    fprintf(file, "# =============================================== "
+                  "Global\n");
+    fprintf(file, "# Language\n");
+    fprintf(file, "%s\n\n", scop->language);
+
+    fprintf(file, "# Context\n");
+
+    osl_relation_pprint_scoplib(file, scop->context, names, 0, 0);
+    fprintf(file, "\n");
+
+    osl_util_print_provided(file,
+        osl_generic_has_URI(scop->parameters, OSL_URI_STRINGS),
+        "Parameters are");
+        
+    if (scop->parameters) {
+      fprintf(file, "# Parameter names\n");
+      osl_strings_print(file, scop->parameters->data);
+    }
+
+    fprintf(file, "\n# Number of statements\n");
+    fprintf(file, "%d\n\n",osl_statement_number(scop->statement));
+
+    osl_statement_pprint_scoplib(file, scop->statement, names);
+
+    if (scop->extension) {
+      fprintf(file, "# =============================================== "
+                    "Options\n");
+      osl_generic_print_options_scoplib(file, scop->extension);
+    }
+    
+    // If necessary, switch back parameter names.
+    if (parameters_backedup) {
+      parameters_backedup = 0;
+      names->parameters = parameters_backup;
+    }
+
+    // If necessary, switch back array names.
+    if (arrays_backedup) {
+      arrays_backedup = 0;
+      osl_strings_free(names->arrays);
+      names->arrays = arrays_backup;
+    }
+
     scop = scop->next;
   }
 
@@ -319,7 +437,7 @@ osl_scop_p osl_scop_pread(FILE * file, osl_interface_p registry,
     //
     // I. START TAG
     //
-    tmp = osl_util_read_uptotag(file, OSL_TAG_START_SCOP);
+    tmp = osl_util_read_uptotag(file, NULL, OSL_URI_SCOP);
     if (tmp == NULL) {
       OSL_debug("no more scop in the file");
       break;
@@ -474,6 +592,39 @@ void osl_scop_free(osl_scop_p scop) {
  *****************************************************************************/
 
 
+/**
+ * osl_scop_add function:
+ * this function adds a scop "scop" at the end of the scop list pointed
+ * by "location".
+ * \param[in,out] location  Address of the first element of the scop list.
+ * \param[in]     scop      The scop to add to the list.
+ */
+void osl_scop_add(osl_scop_p * location, osl_scop_p scop) {
+  while (*location != NULL)
+    location = &((*location)->next);
+
+  *location = scop;
+}
+
+
+/**
+ * osl_scop_number function:
+ * this function returns the number of scops in the scop list
+ * provided as parameter.
+ * \param[in] scop The first element of the scop list.
+ * \return The number of scops in the scop list.
+ */
+int osl_scop_number(osl_scop_p scop) {
+  int number = 0;
+
+  while (scop != NULL) {
+    number++;
+    scop = scop->next;
+  }
+  return number;
+}
+
+
 /**
  * osl_scop_clone function:
  * This functions builds and returns a "hard copy" (not a pointer copy)
@@ -619,6 +770,71 @@ int osl_scop_integrity_check(osl_scop_p scop) {
 }
 
 
+/**
+ * osl_scop_check_compatible_scoplib function:
+ * This function checks that a scop is "well formed". It returns 0 if the
+ * check failed or 1 if no problem has been detected.
+ * \param scop  The scop we want to check.
+ * \return 0 if the integrity check fails, 1 otherwise.
+ */
+int osl_scop_check_compatible_scoplib(osl_scop_p scop) {
+  
+  if (!osl_scop_integrity_check(scop))
+    return 0;
+  if (scop->next != NULL)
+    return 0;
+  if (scop == NULL || scop->statement == NULL)
+    return 1;
+  
+  osl_relation_p domain;
+  osl_statement_p statement;
+  osl_relation_p scattering;
+  int precision = scop->statement->scattering->precision;
+  int i, j;
+  
+  statement = scop->statement;
+  while (statement != NULL) {
+    scattering = statement->scattering;
+    
+    if (scattering->nb_local_dims != 0) {
+      OSL_error("Local dims in scattering matrix");
+      return 0;
+    }
+    
+    domain = statement->domain;
+    while (domain != NULL) {
+      if (domain->nb_local_dims != 0) {
+        OSL_error("Local dims in domain matrix");
+        return 0;
+      }
+      domain = domain->next;
+    }
+    
+    // Check if there is only the -Identity in the output_dims
+    // and the lines MUST be in the right order
+    for (i = 0 ; i < scattering->nb_rows ; i++) {
+      for (j = 0 ; j < scattering->nb_output_dims ; j++) {
+        if (i == j) { // -1
+          if (!osl_int_mone(precision, scattering->m[i][j+1])) {
+            OSL_error("Wrong -Identity");
+            return 0;
+          }
+        } else { // 0
+          if (!osl_int_zero(precision, scattering->m[i][j+1])) {
+            OSL_error("Wrong -Identity");
+            return 0;
+          }
+        }
+      }
+    }
+    
+    statement = statement->next;
+  }
+
+  return 1;
+}
+
+
 /**
  * osl_scop_get_nb_parameters function:
  * this function returns the number of global parameters of a given SCoP.
@@ -656,7 +872,7 @@ void osl_scop_register_extension(osl_scop_p scop, osl_interface_p interface) {
     textual = osl_generic_lookup(scop->extension, interface->URI);
     if (textual != NULL) {
       extension_string = ((osl_textual_p)textual->data)->textual;
-      new = osl_generic_sread(extension_string, interface);
+      new = osl_generic_sread(&extension_string, interface);
       osl_generic_add(&scop->extension, new);
     }
   }
