@@ -86,7 +86,7 @@
  * \param[in] file The (opened) file to read.
  * \param[in] str  Address of an allocated space to store the first line
  *                 that contains useful information.
- * \return The address of the the first useful digit in str.
+ * \return The address of the first useful digit in str.
  */
 char * osl_util_skip_blank_and_comments(FILE * file, char * str) {
   char * start;
@@ -157,7 +157,7 @@ int osl_util_read_int(FILE * file, char ** str) {
     osl_util_sskip_blank_and_comments(str);
     
     // Build the chain to analyze.
-    while (**str && !isspace(**str) && **str != '\n')
+    while (**str && !isspace(**str) && **str != '\n' && **str != '#')
       s[i++] = *((*str)++);
     s[i] = '\0';
     if (sscanf(s, "%d", &res) != 1)
@@ -168,13 +168,97 @@ int osl_util_read_int(FILE * file, char ** str) {
 }
 
 
+/**
+ * osl_util_read_string function:
+ * reads a string on the input 'file' or the input string 'str' depending on
+ * which one is not NULL (exactly one of them must not be NULL).
+ * \param[in]     file The file where to read a string (if not NULL).
+ * \param[in,out] str  The string where to read a string (if not NULL). This
+ *                     pointer is updated to reflect the read and points
+ *                     after the string in the input string.
+ * \return The string that has been read.
+ */
+char * osl_util_read_string(FILE * file, char ** str) {
+  char s[OSL_MAX_STRING], * start;
+  char * res;
+  int i = 0;
+
+  if ((file != NULL && str != NULL) || (file == NULL && str == NULL))
+    OSL_error("one and only one of the two parameters can be non-NULL");
+
+  OSL_malloc(res, char *, OSL_MAX_STRING * sizeof(char));
+  if (file != NULL) {
+    // Parse from a file.
+    start = osl_util_skip_blank_and_comments(file, s);
+    if (sscanf(start, " %s", res) != 1)
+      OSL_error("a string was expected");
+  }
+  else {
+    // Parse from a string.
+    // Skip blank/commented lines.
+    osl_util_sskip_blank_and_comments(str);
+    
+    // Build the chain to analyze.
+    while (**str && !isspace(**str) && **str != '\n' && **str != '#')
+      s[i++] = *((*str)++);
+    s[i] = '\0';
+    if (sscanf(s, "%s", res) != 1)
+      OSL_error("a string was expected");
+  }
+
+  OSL_realloc(res, char *, strlen(res) + 1);
+  return res;
+}
+
+
+/**
+ * osl_util_read_line function:
+ * reads a line on the input 'file' or the input string 'str' depending on
+ * which one is not NULL (exactly one of them must not be NULL). A line
+ * is defined as the array of characters before the comment tag or the end of
+ * line (it may include spaces).
+ * \param[in]     file The file where to read a line (if not NULL).
+ * \param[in,out] str  The string where to read a line (if not NULL). This
+ *                     pointer is updated to reflect the read and points
+ *                     after the line in the input string.
+ * \return The line that has been read.
+ */
+char * osl_util_read_line(FILE * file, char ** str) {
+  char s[OSL_MAX_STRING], * start;
+  char * res;
+  int i = 0;
+
+  if ((file != NULL && str != NULL) || (file == NULL && str == NULL))
+    OSL_error("one and only one of the two parameters can be non-NULL");
+
+  OSL_malloc(res, char *, OSL_MAX_STRING * sizeof(char));
+  if (file != NULL) {
+    // Parse from a file.
+    start = osl_util_skip_blank_and_comments(file, s);
+    while (*start && *start != '\n' && *start != '#' && i < OSL_MAX_STRING)
+      res[i++] = *start++;
+  }
+  else {
+    // Parse from a string.
+    osl_util_sskip_blank_and_comments(str);
+    while (**str && **str != '\n' && **str != '#' && i < OSL_MAX_STRING)
+      res[i++] = *((*str)++);
+  }
+
+  res[i] = '\0';
+  OSL_realloc(res, char *, strlen(res) + 1);
+  return res;
+}
+
+
 /**
  * osl_util_read_int internal function:
  * reads a tag (the form of a tag with name "name" is \<name\>) on the input
  * 'file' or the input string 'str' depending on which one is not NULL (exactly
  * one of them must not be NULL). It returns the name of the tag (thus without
  * the < and > as a string. Note that in the case of an ending tag, e.g.,
- * \</foo\>, the slash is returned as a part of the name, e.g., /foo.
+ * \</foo\>, the slash is returned as a part of the name, e.g., /foo. If no
+ * tag is found the function returns NULL.
  * \param[in]     file The file where to read a tag (if not NULL).
  * \param[in,out] str  The string where to read a tag (if not NULL). This
  *                     pointer is updated to reflect the read and points
@@ -198,6 +282,11 @@ char * osl_util_read_tag(FILE * file, char ** str) {
     osl_util_sskip_blank_and_comments(str);
   }
 
+  // If the end of the input has been reached, return NULL.
+  if (((file != NULL) && (feof(file))) ||
+      ((str  != NULL) && (**str == '\0')))
+    return NULL;
+  
   // Pass the starting '<'.
   if (**str != '<')
     OSL_error("a \"<\" to start a tag was expected");
@@ -230,31 +319,40 @@ char * osl_util_read_tag(FILE * file, char ** str) {
 
 
 /**
- * osl_util_read_uptotag function:
- * this function reads a file up to a given tag (the tag is read) or the
- * end of file. It puts everything it reads, except the tag, in a string 
- * which is returned. However ot returns NULL is the tag is not found.
- * \param[in] file The file where to read the tail.
- * \param[in] tag  The tag which, when reached, stops the file reading.
- * \return The string that has been read from the file.
+ * osl_util_read_uptoflag function:
+ * this function reads a string up to a given flag (the flag is read)
+ * on the input 'file' or the input string 'str' depending on which one is
+ * not NULL (exactly one of them must not be NULL) and returns that string
+ * without the flag. It returns NULL if the flag is not found.
+ * \param[in]     file The file where to read up to flag (if not NULL).
+ * \param[in,out] str  The string where to read up to flag (if not NULL). This
+ *                     pointer is updated to reflect the read and points
+ *                     after the flag in the input string.
+ * \param[in]     flag The flag which, when reached, stops the reading.
+ * \return The string that has been read.
  */
-char * osl_util_read_uptotag(FILE * file, char * tag) {
+char * osl_util_read_uptoflag(FILE * file, char ** str, char * flag) {
   int high_water_mark = OSL_MAX_STRING;
   int nb_chars = 0;
-  int lentag = strlen(tag);
-  int tag_found = 0;
+  int lenflag = strlen(flag), lenstr;
+  int flag_found = 0;
   char * res;
 
+  if ((file != NULL && str != NULL) || (file == NULL && str == NULL))
+    OSL_error("one and only one of the two parameters can be non-NULL");
+  
   OSL_malloc(res, char *, high_water_mark * sizeof(char));
 
-  // - Copy everything to the res string.
-  while (!feof(file)) {
-    res[nb_chars] = fgetc(file); 
-    nb_chars++;
+  // Copy everything to the res string.
+  if (str != NULL)
+    lenstr = strlen(*str);
+  while (((str  != NULL) && (nb_chars != lenstr)) ||
+         ((file != NULL) && (!feof(file)))) {
+    res[nb_chars++] = (str != NULL) ? *((*str)++) : fgetc(file);
 
-    if ((nb_chars >= lentag) &&
-        (!strncmp(&res[nb_chars - lentag], tag, lentag))) {
-      tag_found = 1;
+    if ((nb_chars >= lenflag) &&
+        (!strncmp(&res[nb_chars - lenflag], flag, lenflag))) {
+      flag_found = 1;
       break;
     }
 
@@ -264,35 +362,59 @@ char * osl_util_read_uptotag(FILE * file, char * tag) {
     }
   }
 
-  if (!tag_found) {
-    OSL_debug("tag was not found, end of file reached");
+  if (!flag_found) {
+    OSL_debug("flag was not found, end of input reached");
     free(res);
     return NULL;
   }
 
   // - 0-terminate the string.
-  OSL_realloc(res, char *, (nb_chars - strlen(tag) + 1) * sizeof(char));
-  res[nb_chars - strlen(tag)] = '\0';
+  OSL_realloc(res, char *, (nb_chars - strlen(flag) + 1) * sizeof(char));
+  res[nb_chars - strlen(flag)] = '\0';
 
   return res;
 }
 
 
 /**
- * osl_util_read_uptoendtag function:
- * this function reads a file up to a given end tag (this end tag is read)
- * or the end of file. The name of the tag is provided as parameter (hence
- * without the starting "</" end the closing ">"). It puts everything it reads
- * in a string which is returned.
- * \param[in] file The file where to read the tail.
- * \param[in] name The name of the end tag to the file reading.
+ * osl_util_read_uptotag function:
+ * this function reads a string up to a given (start) tag (the tag is read)
+ * on the input 'file' or the input string 'str' depending on which one is
+ * not NULL (exactly one of them must not be NULL) and returns that string
+ * without the tag. It returns NULL if the tag is not found.
+ * \param[in]     file The file where to read up to tag (if not NULL).
+ * \param[in,out] str  The string where to read up to tag (if not NULL).
+ *                     This pointer is updated to reflect the read and points
+ *                     after the tag in the input string.
+ * \param[in]     name The name of the tag to the file reading.
  * \return The string that has been read from the file.
  */
-char * osl_util_read_uptoendtag(FILE * file, char * name) {
-  char tag[strlen(name) + 4];
+char * osl_util_read_uptotag(FILE * file, char ** str, char * name) {
+  char tag[strlen(name) + 3];
   
-  sprintf(tag, "</%s>", name);
-  return osl_util_read_uptotag(file, tag);
+  sprintf(tag, "<%s>", name);
+  return osl_util_read_uptoflag(file, str, tag);
+}
+
+
+/**
+ * osl_util_read_uptoendtag function:
+ * this function reads a string up to a given end tag (the end tag is read)
+ * on the input 'file' or the input string 'str' depending on which one is
+ * not NULL (exactly one of them must not be NULL) and returns that string
+ * without the end tag. It returns NULL if the end tag is not found.
+ * \param[in]     file The file where to read up to end tag (if not NULL).
+ * \param[in,out] str  The string where to read up to end tag (if not NULL).
+ *                     This pointer is updated to reflect the read and points
+ *                     after the end tag in the input string.
+ * \param[in]     name The name of the end tag to the file reading.
+ * \return The string that has been read from the file.
+ */
+char * osl_util_read_uptoendtag(FILE * file, char ** str, char * name) {
+  char endtag[strlen(name) + 4];
+  
+  sprintf(endtag, "</%s>", name);
+  return osl_util_read_uptoflag(file, str, endtag);
 }
 
 
@@ -389,10 +511,17 @@ int osl_util_get_precision() {
       precision = OSL_PRECISION_SP;
     else if (!strcmp(precision_env, OSL_PRECISION_ENV_DP))
       precision = OSL_PRECISION_DP;
-    else if (!strcmp(precision_env, OSL_PRECISION_ENV_MP))
+    else if (!strcmp(precision_env, OSL_PRECISION_ENV_MP)) {
+#ifndef OSL_GMP_IS_HERE
+      OSL_warning("$OSL_PRECISION says GMP but osl not compiled with "
+                  "GMP support, switching to double precision");
+      precision = OSL_PRECISION_DP;
+#else
       precision = OSL_PRECISION_MP;
+#endif
+    }
     else
-      OSL_warning("bad precision environment value");
+      OSL_warning("bad OSL_PRECISION environment value, see osl's manual");
   }
 
   return precision;
