@@ -115,6 +115,9 @@ int test_file(char * input_name, int verbose) {
   }
   input_scop = osl_scop_read(input_file);
   fclose(input_file);
+  
+  if (verbose)
+    printf("- reading file succeeded\n");
 
   // PART II. Clone and test.
   cloned_scop = osl_scop_clone(input_scop);
@@ -155,7 +158,7 @@ int test_file(char * input_name, int verbose) {
     printf("- dumping failed\n");
 
   // PART IV. Report.
-  if (equal = (cloning + dumping > 0) ? 1 : 0)
+  if (equal = (cloning + dumping > 1) ? 1 : 0)
     printf("Success :-)\n");
   else
     printf("Failure :-(\n");
